#!/usr/bin/env python
"""Unit tests for isbn_hyphenate.py"""

from __future__ import absolute_import, print_function, unicode_literals
import isbn_hyphenate
import unittest

class KnownValues(unittest.TestCase):
    knownValues = ( "99921-58-10-7",
                    "9971-5-0210-0",
                    "960-425-059-0",
                    "80-902734-1-6",
                    "85-359-0277-5",
                    "1-84356-028-3",
                    "0-684-84328-5",
                    "0-8044-2957-X",
                    "0-85131-041-9",
                    "0-943396-04-2",
                    "0-9752298-0-X",
                    "978-0-321-53496-5",
                    "978-3-16-148410-0",
                    "1-4028-9462-7",
                    "978-1-4028-9462-6",
                    "978-99953-838-2-4",
                    "978-99930-75-89-9",
                    "978-1-59059-356-1",
                  )

    def test_hyphenating_known_values(self):
        for with_hyphens in self.knownValues:
            without_hyphens = with_hyphens.replace('-', '')
            self.assertEqual(isbn_hyphenate.hyphenate(without_hyphens), with_hyphens)
            
    def test_try_hyphenating_known_values(self):
        for with_hyphens in self.knownValues:
            without_hyphens = with_hyphens.replace('-', '')
            self.assertEqual(isbn_hyphenate.try_hyphenate(without_hyphens), with_hyphens)


class BadInput(unittest.TestCase):
    def test_bad_characters(self):
        self.assertRaises(isbn_hyphenate.IsbnMalformedError, isbn_hyphenate.hyphenate, "fghdf hdfjhfgj")
    def test_try_bad_characters(self):
        self.assertRaises(isbn_hyphenate.IsbnMalformedError, isbn_hyphenate.try_hyphenate, "fghdf hdfjhfgj")

    def test_too_short(self):
        self.assertRaises(isbn_hyphenate.IsbnMalformedError, isbn_hyphenate.hyphenate, "12345")

    def test_unknown_prefix(self):
        self.assertRaises(isbn_hyphenate.IsbnUnableToHyphenateError, isbn_hyphenate.hyphenate, "9751402894626")

    def test_unused_prefix(self):
        self.assertRaises(isbn_hyphenate.IsbnUnableToHyphenateError, isbn_hyphenate.hyphenate, "9786500042626")

    def test_unknown_prefix2(self):
        self.assertRaises(isbn_hyphenate.IsbnUnableToHyphenateError, isbn_hyphenate.hyphenate, "9789999999626")

    def test_unused_prefix2(self):                                          
        self.assertRaises(isbn_hyphenate.IsbnUnableToHyphenateError, isbn_hyphenate.hyphenate, "9789927512300")

    def test_try_unknown_prefix(self):                                          
        isbn_unknown = "9751402894626"
        self.assertEqual(isbn_hyphenate.try_hyphenate(isbn_unknown), isbn_unknown)

    def test_empty_input(self):                                          
        self.assertRaises(isbn_hyphenate.IsbnMalformedError, isbn_hyphenate.hyphenate, "")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from __future__ import absolute_import, print_function, unicode_literals
import xml.dom.minidom
import sys

def get_text(startnode):
    text = ""
    for node in startnode.childNodes:
        if node.nodeType == node.TEXT_NODE:
            text = text + node.data
    return text

def parse_prefix_length_map(dom):
    prefix = get_text(dom.getElementsByTagName("Prefix")[0])
    length_map = []
    for rule in dom.getElementsByTagName("Rules")[0].getElementsByTagName("Rule"):
        rule_range = get_text(rule.getElementsByTagName("Range")[0])
        length = get_text(rule.getElementsByTagName("Length")[0])
        (range_min, range_max) = rule_range.strip().split('-')
        range_to_length = { 'min'    : int(range_min),
                            'max'    : int(range_max),
                            'length' : int(length)
                            }
        length_map.append(range_to_length)
    return (prefix, length_map)

if len(sys.argv) != 2:
    print("Usage:\n./isbn_xml2py.py RangeMessage.xml > isbn_lengthmaps.py", file=sys.stderr)
    sys.exit(2)

def main():
    xmlfile = sys.argv[1]
    dom = xml.dom.minidom.parse(xmlfile)

    ISBN_range_message = dom.getElementsByTagName("ISBNRangeMessage")[0]

    groups_length = {}
    for ean in ISBN_range_message.getElementsByTagName("EAN.UCCPrefixes")[0].getElementsByTagName("EAN.UCC"):
        (prefix, length_map) = parse_prefix_length_map(ean)
        groups_length[prefix] = length_map

    publisher_length = {}
    for group in ISBN_range_message.getElementsByTagName("RegistrationGroups")[0].getElementsByTagName("Group"):
        (prefix, length_map) = parse_prefix_length_map(group)
        publisher_length[prefix] = length_map

    message_serial = get_text(ISBN_range_message.getElementsByTagName("MessageSerialNumber")[0])
    message_date = get_text(ISBN_range_message.getElementsByTagName("MessageDate")[0])

    print('# Generated from RangeMessage.xml with isbn_xml2py.py')
    print('# Available from http://www.isbn-international.org/agency?rmxml=1')
    print('# MessageDate: ' + message_date)
    print('# MessageSerialNumber: ' + message_serial)
    print('groups_length = ' + repr(groups_length).replace("u'", "'"))
    print('publisher_length = ' + repr(publisher_length).replace("u'", "'"))

main()



from __future__ import absolute_import, print_function, unicode_literals
import re
import isbn_lengthmaps

class IsbnError(Exception): pass

# ISBN is malformed, such as too short, too long or contains invalid characters
class IsbnMalformedError(IsbnError): pass

# Unable to hyphenate is ISBN. Either it does not exist, or
# the ranges information used by this library is out of date
class IsbnUnableToHyphenateError(IsbnError): pass


def hyphenate(input_data):
    """Add hyphens to an International Standard Book Number (ISBN)
    
    >>> hyphenate('1843560283')
    '1-84356-028-3'
    
    10 or 13 digit ISBN are hyphenated. Hyphens and whitespace in the input
    is stipped. 
    
    This function uses a database of ISBN ranges, so might not work for
    ISBNs in recently allocated ranges.
    """

    without_hyphens = re.sub('[\s-]', '', input_data)
    
    if re.match('[^0-9X]', without_hyphens):
        raise IsbnMalformedError("Must only contain digits and/or and X")
    
    with_hyphens = ''
    
    GS1_prefix = None
    if len(without_hyphens) == 13:
        GS1_prefix = without_hyphens[:3]
        with_hyphens = GS1_prefix + '-'
        without_hyphens = without_hyphens[3:]
    elif len(without_hyphens) == 10:
        GS1_prefix = '978'
    else:
        raise IsbnMalformedError("Length must be 10 or 13")
    
    first7 = int(without_hyphens[:7])
    group_prefix_length = None
    if GS1_prefix not in isbn_lengthmaps.groups_length:
        raise IsbnUnableToHyphenateError("GS1 prefix %s not recognized" % GS1_prefix)
    for cur_range in isbn_lengthmaps.groups_length[GS1_prefix]:
        if cur_range['min'] <= first7 <= cur_range['max']:
            group_prefix_length = cur_range['length']
            break
    
    if group_prefix_length is None:
        raise IsbnUnableToHyphenateError("Not in any recognized group range")
    elif group_prefix_length == 0:
        raise IsbnUnableToHyphenateError("Group range is unused")
    
    group_prefix = without_hyphens[:group_prefix_length]
    without_hyphens = without_hyphens[group_prefix_length:]
    with_hyphens += group_prefix + '-'
    
    first7 = int(without_hyphens[:7].ljust(7, '0'))
    publisher_length = None
    GS1_and_group = GS1_prefix + '-' + group_prefix
    if GS1_and_group not in isbn_lengthmaps.publisher_length:
        raise IsbnUnableToHyphenateError("Prefix %s not recognized" % GS1_and_group)
    for cur_range in isbn_lengthmaps.publisher_length[GS1_and_group]:
        if cur_range['min'] <= first7 <= cur_range['max']:
            publisher_length = cur_range['length']
            break

    if publisher_length is None:
        raise IsbnUnableToHyphenateError("Not in any recognized publisher range")
    elif publisher_length == 0:
        raise IsbnUnableToHyphenateError("Publisher range is unused")
    
    publisher = without_hyphens[:publisher_length]
    without_hyphens = without_hyphens[publisher_length:]
    with_hyphens += publisher + '-'
    
    book_id = without_hyphens[:-1]
    check_digit = without_hyphens[-1:]
    with_hyphens += book_id + '-' + check_digit
    
    return with_hyphens
    

def try_hyphenate(isbn):
    try:
        return hyphenate(isbn)
    except IsbnUnableToHyphenateError:
        return isbn
        



from distutils.core import setup

setup(
    name = 'isbn_hyphenate',
    packages = ['isbn_hyphenate'],
    version = '1',
    description = 'a Python library to add hyphens in the right place to an ISBN (International Standard Book Number)',
    author = 'Tor Klingberg',
    author_email = 'tor.klingberg@gmail.com',
    url = 'https://github.com/TorKlingberg/isbn_hyphenate',
    license = "LGPL",
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Text Processing :: General',
        'Topic :: Printing',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
